package com.fr.plugin.pielinecomb.ui;

import com.fr.config.predefined.ColorFillStyle;
import com.fr.design.gui.frpane.UINumberDragPane;
import com.fr.design.gui.ibutton.UIButtonGroup;
import com.fr.design.gui.icheckbox.UICheckBox;
import com.fr.design.gui.ilable.UILabel;
import com.fr.design.gui.ispinner.UISpinner;
import com.fr.design.i18n.Toolkit;
import com.fr.design.layout.TableLayout;
import com.fr.design.layout.TableLayoutHelper;
import com.fr.design.style.color.ColorSelectBox;
import com.fr.extended.chart.ExtendedScrollPane;
import com.fr.json.JSONObject;
import com.fr.plugin.pielinecomb.PieLineCombChart;
import com.fr.plugin.pielinecomb.comp.CustomChartFillStylePane;
import com.fr.stable.StringUtils;
import com.fr.van.chart.designer.TableLayout4VanChartHelper;

import javax.swing.*;
import java.awt.*;

public class PieLineCombStyleBgPane extends ExtendedScrollPane<PieLineCombChart> {

    private ColorSelectBox bgColor;
    private UINumberDragPane bgOpacity;
    private UICheckBox isShowZoomTool;

    private UIButtonGroup<Integer> chartType;
    private CustomChartFillStylePane fillStylePane;
    private ColorSelectBox gridXColor;

    private ColorSelectBox gridYColor;
    private UISpinner fixHeight;
    private BgGridCondListPane bgGridListPane;


    @Override
    protected JPanel createContentPane() {
        return new ContentPane();
    }

    @Override
    public void populateBean(PieLineCombChart chart) {
        JSONObject chartConf = chart.getBgConf();

        this.bgColor.setSelectObject(MapUtil.toColor(chartConf.getString("bgColor")));
        this.bgOpacity.populateBean(chartConf.getDouble("bgOpacity"));
        this.isShowZoomTool.setSelected(chartConf.getBoolean("isShowZoomTool"));
        this.gridXColor.setSelectObject(MapUtil.toColor(chartConf.getString("gridXColor")));
        this.gridYColor.setSelectObject(MapUtil.toColor(chartConf.getString("gridYColor")));
        this.fixHeight.setValue(chartConf.getDouble("fixHeight"));
        this.chartType.setSelectedItem(chartConf.getInt("chartType"));
        this.bgGridListPane.populateStyle(chart);

        String colors = chartConf.getString("gridXColorList");
        if (StringUtils.isNotEmpty(colors)) {
            java.util.List<Color> colorList = new java.util.ArrayList<Color>();
            String[] colorArr = colors.split("\\|");
            for (int i = 0; i < colorArr.length; i++) {
                colorList.add(MapUtil.toColor(colorArr[i]));
            }
            ColorFillStyle fillStyle = new ColorFillStyle();
            fillStyle.setColorStyle(1);
            fillStyle.setColorList(colorList);
            fillStyle.setCustomFillStyle(chartConf.getBoolean("gridXColorCustom"));
            fillStyle.setFillStyleName(chartConf.getString("gridXColorName"));
            this.fillStylePane.populateBean(fillStyle);
        }

    }

    @Override
    public void updateBean(PieLineCombChart chart) {
        JSONObject chartConf = new JSONObject();

        chartConf.put("bgColor", MapUtil.toHex(this.bgColor.getSelectObject()));
        chartConf.put("bgOpacity", this.bgOpacity.updateBean());
        chartConf.put("isShowZoomTool", this.isShowZoomTool.isSelected());
        chartConf.put("gridXColor", MapUtil.toHex(this.gridXColor.getSelectObject()));
        chartConf.put("gridYColor", MapUtil.toHex(this.gridYColor.getSelectObject()));
        chartConf.put("fixHeight", this.fixHeight.getValue());
        chartConf.put("chartType", chartType.getSelectedItem());
        this.bgGridListPane.updateStyle(chart);

        ColorFillStyle fillStyle = this.fillStylePane.updateBean();
        java.util.List<Color> colorList = fillStyle.getColorList();
        StringBuffer colorStr = new StringBuffer();
        for (int i = 0; i < colorList.size(); i++) {
            if (i > 0) {
                colorStr.append("|");
            }
            String tmpColor = MapUtil.toHex(colorList.get(i));
            colorStr.append(tmpColor);
        }
        chartConf.put("gridXColorList", colorStr.toString());
        chartConf.put("gridXColorName", fillStyle.getFillStyleName());
        chartConf.put("gridXColorCustom", fillStyle.isCustomFillStyle());

        chart.setBgConf(chartConf);
    }

    @Override
    protected String title4PopupWindow() {
        return Toolkit.i18nText("Plugin-Pielinecomb-StyleBgTitle");
    }

    private class ContentPane extends JPanel {
        public ContentPane() {
            initComponents();
        }

        private void initComponents() {
            this.setLayout(new BorderLayout());

            JPanel bgPane = createBgPane();
            JPanel xPane = createBgXPane();
            JPanel yPane = createBgYPane();

            double p = TableLayout.PREFERRED;
            double f = TableLayout.FILL;
            Component[][] acomponents = new Component[][]{
                    new Component[]{bgPane},
                    new Component[]{xPane},
                    new Component[]{yPane}
            };

            JPanel panel = TableLayoutHelper.createTableLayoutPane(acomponents, new double[]{p, p, p, p}, new double[]{f});

            this.add(panel,BorderLayout.CENTER);

            this.setVisible(true);

        }

    }

    private JPanel createBgYPane() {
        this.gridYColor = new ColorSelectBox(100);
        this.fixHeight = new UISpinner(0, 999999999, 1, 100);
        this.bgGridListPane = new BgGridCondListPane();

        double p = TableLayout.PREFERRED;
        double f = TableLayout.FILL;
        Component[][] comp = new Component[][]{
                {null, null},
                new Component[]{new UILabel(Toolkit.i18nText("Plugin-Pielinecomb-fixHeight")), this.fixHeight},
                new Component[]{new UILabel(Toolkit.i18nText("Plugin-Pielinecomb-lineColor")), this.gridYColor}
        };
        JPanel gridPane = TableLayout4VanChartHelper.createGapTableLayoutPane(comp, new double[]{p, p, p}, new double[]{p, f});

        Component[][] comps = new Component[][]{
                new Component[]{gridPane},
                new Component[]{this.bgGridListPane}
        };

        JPanel pane = TableLayout4VanChartHelper.createGapTableLayoutPane(comps, new double[]{p, p, p}, new double[]{f});

        return TableLayout4VanChartHelper.createExpandablePaneWithTitle(Toolkit.i18nText("Plugin-Pielinecomb-BgYTitleStyle"), pane);
    }

    private JPanel createBgXPane() {
        chartType = new UIButtonGroup<Integer>(new String[]{
                Toolkit.i18nText("Plugin-Pielinecomb-Typepie"),
                Toolkit.i18nText("Plugin-Pielinecomb-Typeline")
        },
                new Integer[]{1, 2}
        );
        chartType.setSelectedItem(1);

        this.gridXColor = new ColorSelectBox(100);
        this.fillStylePane = new CustomChartFillStylePane();

        double p = TableLayout.PREFERRED;
        double f = TableLayout.FILL;
        Component[][] comp = new Component[][]{
                {null, null},
                new Component[]{new UILabel(Toolkit.i18nText("Plugin-Pielinecomb-bgChartType")), this.chartType},
                new Component[]{new UILabel(Toolkit.i18nText("Plugin-Pielinecomb-lineColor")), this.gridXColor}
        };
        JPanel gridPane = TableLayout4VanChartHelper.createGapTableLayoutPane(comp, new double[]{p, p, p}, new double[]{p, f});

        Component[][] comps = new Component[][]{
                new Component[]{gridPane},
                new Component[]{this.fillStylePane}
        };

        JPanel pane = TableLayout4VanChartHelper.createGapTableLayoutPane(comps, new double[]{p, p, p}, new double[]{f});

        return TableLayout4VanChartHelper.createExpandablePaneWithTitle(Toolkit.i18nText("Plugin-Pielinecomb-BgXTitleStyle"), pane);
    }

    private JPanel createBgPane() {
        this.bgColor = new ColorSelectBox(100);
        this.bgOpacity = new UINumberDragPane(0, 100);
        this.isShowZoomTool = new UICheckBox(Toolkit.i18nText("Plugin-Pielinecomb-isShowZoomTool"));

        double p = TableLayout.PREFERRED;
        double f = TableLayout.FILL;
        Component[][] comp = new Component[][]{
                {null, null},
                new Component[]{new UILabel(Toolkit.i18nText("Plugin-Pielinecomb-titleBgColor")), this.bgColor},
                new Component[]{new UILabel(Toolkit.i18nText("Plugin-Pielinecomb-titleBgOpacity")), this.bgOpacity},
                new Component[]{null, this.isShowZoomTool}
        };
        JPanel pane = TableLayout4VanChartHelper.createGapTableLayoutPane(comp, new double[]{p, p, p, p}, new double[]{p, f});
        return TableLayout4VanChartHelper.createExpandablePaneWithTitle(Toolkit.i18nText("Plugin-Pielinecomb-bgExpand"), pane);
    }

}
