package com.fr.plugin.pielinecomb.vo;

import com.fr.js.NameJavaScriptGroup;
import com.fr.stable.FCloneable;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLReadable;
import com.fr.stable.xml.XMLableReader;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author duan.jingliang
 * @date 2022/10/23
 */
public class HyperLinkObject implements XMLReadable, Serializable, FCloneable {

    public static final String XML_TAG_PIE = "CustomHyperLinkPie";
    public static final String XML_TAG_LINE = "CustomHyperLinkLine";

    public static final String CHART_TYPE_PIE = "pie";
    public static final String CHART_TYPE_LINE = "line";

    private Map<String, NameJavaScriptGroup> linkMap = new HashMap<>();

    public NameJavaScriptGroup getNameJavaScriptGroup(String name) {
        return this.linkMap.get(name);
    }

    public void setNameJavaScriptGroup(String name, NameJavaScriptGroup nameGroup) {
        this.linkMap.put(name, nameGroup);
    }

    @Override
    public void readXML(XMLableReader xmLableReader) {

        if (xmLableReader.isChildNode()) {
            String tagName = xmLableReader.getTagName();
            if (XML_TAG_PIE.equals(tagName)) {
                NameJavaScriptGroup obj = new NameJavaScriptGroup();
                xmLableReader.readXMLObject(obj);
                this.linkMap.put(CHART_TYPE_PIE, obj);
            } else if (XML_TAG_LINE.equals(tagName)) {
                NameJavaScriptGroup obj = new NameJavaScriptGroup();
                xmLableReader.readXMLObject(obj);
                this.linkMap.put(CHART_TYPE_LINE, obj);
            }
        }
    }

    public void writeXML(XMLPrintWriter xmlWriter) {
        if (null != this.linkMap && !this.linkMap.isEmpty()) {
            NameJavaScriptGroup pieObj = this.linkMap.get(CHART_TYPE_PIE);
            if (null != pieObj) {
                xmlWriter.startTAG(XML_TAG_PIE);
                pieObj.writeXML(xmlWriter);
                xmlWriter.end();
            }

            NameJavaScriptGroup lineObj = this.linkMap.get(CHART_TYPE_LINE);
            if (null != lineObj) {
                xmlWriter.startTAG(XML_TAG_LINE);
                lineObj.writeXML(xmlWriter);
                xmlWriter.end();
            }
        }
    }


    @Override
    public Object clone() throws CloneNotSupportedException {
        HyperLinkObject obj = (HyperLinkObject)super.clone();
        Map<String, NameJavaScriptGroup> newLinkMap = new HashMap<>();
        if (null != this.linkMap && !this.linkMap.isEmpty()) {
            Iterator<String> it = this.linkMap.keySet().iterator();
            while(it.hasNext()) {
                String name = it.next();
                newLinkMap.put(name, (NameJavaScriptGroup)this.linkMap.get(name).clone());
            }
            obj.linkMap = newLinkMap;
        }
        return obj;
    }

    @Override
    public boolean equals(Object srcObj) {
        if (!(srcObj instanceof HyperLinkObject)) {
            return false;
        } else {
            HyperLinkObject srcHyperLink = (HyperLinkObject)srcObj;
            if (this.linkMap.size() != srcHyperLink.linkMap.size()) {
                return false;
            } else {
                if ((null != this.linkMap.get(CHART_TYPE_PIE) && null == srcHyperLink.linkMap.get(CHART_TYPE_PIE)) ||
                        (null == this.linkMap.get(CHART_TYPE_PIE) && null != srcHyperLink.linkMap.get(CHART_TYPE_PIE))) {
                    return false;
                }
                if (null != this.linkMap.get(CHART_TYPE_PIE)
                    && null != srcHyperLink.linkMap.get(CHART_TYPE_PIE)
                    && !this.linkMap.get(CHART_TYPE_PIE).equals(srcHyperLink.linkMap.get(CHART_TYPE_PIE))) {
                    return false;
                }

                if ((null != this.linkMap.get(CHART_TYPE_LINE) && null == srcHyperLink.linkMap.get(CHART_TYPE_LINE)) ||
                        (null == this.linkMap.get(CHART_TYPE_LINE) && null != srcHyperLink.linkMap.get(CHART_TYPE_LINE))) {
                    return false;
                }
                if (null != this.linkMap.get(CHART_TYPE_LINE)
                        && null != srcHyperLink.linkMap.get(CHART_TYPE_LINE)
                        && !this.linkMap.get(CHART_TYPE_LINE).equals(srcHyperLink.linkMap.get(CHART_TYPE_LINE))) {
                    return false;
                }
            }
            return true;
        }
    }
}
