(function () {
    var system = require('system');
    var fs = require('fs');
    var config = {
        FR: 'fr.js',
        JQUERY: 'jquery.js',
        ECHARTS: 'echarts.js',
        CHARTWRAPPER: 'chartwrapper.js',
        CHARTDATA: 'chartdata.js',
        DEFAULT_WIDTH: '150',
        DEFAULT_HEIGHT: '150'
    }, parseParams, render;

    // 处理参数
    parseParams = function () {
        var map = {}, i, key;
        if (system.args.length < 2) {
            phantom.exit();
        }
        for (i = 0; i < system.args.length; i += 1) {
            if (system.args[i].charAt(0) === '-') {
                key = system.args[i].substr(1, i.length);
                if (key === 'infile') {
                    // get string from file
                    // force translate the key from infile to options.
                    key = 'options';
                    try {
                        map[key] = fs.read(system.args[i + 1]).replace(/^\s+/, '');
                    } catch (e) {
                        console.log('Error: cannot find file, ' + system.args[i + 1]);
                        phantom.exit();
                    }
                } else {
                    map[key] = system.args[i + 1].replace(/^\s+/, '');
                }
            }
        }
        return map;
    };

    render = function (params) {
        var page = require('webpage').create(), createChart;

        page.onConsoleMessage = function (msg) {
            console.log(msg);
        };

        page.onAlert = function (msg) {
            console.log(msg);
        };

        createChart = function (chartname, width, height) {

            // we render the image, so we need set background to white.
            //$(document.body).css('backgroundColor', 'white');
            var container = $("<div>").appendTo(document.body);
            container.attr('id', 'container');
            container.css({
                width: width,
                height: height
            });
            // render the chart

            var chart = new window[chartname];
            var chartInst = chart._init(container[0], window.chartOptions);
            return chart.getChartImage() || chartInst.getDataURL();

        };

        // parse the params
        page.open("about:blank", function (status) {
            // inject the dependency js
            page.injectJs(config.FR);
            page.injectJs(config.JQUERY);
            page.injectJs(config.ECHARTS);
            page.injectJs(config.CHARTDATA);
            page.injectJs(config.CHARTWRAPPER);

            // create the chart
            var base64 = page.evaluate(createChart, params.chartname, params.width, params.height);

            setTimeout(function() {
                base64 = page.evaluate(function() {
                    return window.PieLineCombChartInst.getChartImage();
                });

                console.log(base64);
                // exit
                phantom.exit();

            }, 1000);


        });
    };

    // get the args
    var params = parseParams();

    // render the image
    render(params);
}());